-- name: [CS] Mr. Virtual
-- description: Mr. V's official SM64COOPDX mod! Released as part of The Week Of V!

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
]]

local TEXT_MOD_NAME = "Mr. Virtual"

-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MODEL_MRV = smlua_model_util_get_id("mrv_geo") -- Located in "actors"

local MRV_LIFE_ICON = get_texture_info("mrv-hud") -- Located in "textures"
--local TEX_CUSTOM_STAR_ICON = get_texture_info("exclamation-icon") -- Located in "textures"

-- All Located in "actors"
local MRV_CAPTABLE = {
    normal = smlua_model_util_get_id("mrv_cap_geo"),
    wing = smlua_model_util_get_id("mrv_wing_cap_geo"),
    metal = smlua_model_util_get_id("mrv_metal_cap_geo"),
    metalWing = smlua_model_util_get_id("mrv_winged_metal_cap_geo"),
}

--local E_MODEL_CUSTOM_STAR = smlua_model_util_get_id("vb_geo") -- Located in "actors"

local PALETTE_CHAR = {
    [PANTS]  = "FF0600",
    [SHIRT]  = "8D0000",
    [GLOVES] = "000000",
    [SHOES]  = "000000",
    [HAIR]   = "000000",
    [SKIN]   = "FFFFFF",
    [CAP]    = "000000",
    [EMBLEM]    = "000000",
}

--[[
-- All Located in "textures"
local healthMeter = {
    label = {
        left = get_texture_info("template-hp-back-left"),
        right = get_texture_info("template-hp-back-right"),
    },
    pie = {
        [1] = get_texture_info("template-hp-pie-1"),
        [2] = get_texture_info("template-hp-pie-2"),
        [3] = get_texture_info("template-hp-pie-3"),
        [4] = get_texture_info("template-hp-pie-4"),
        [5] = get_texture_info("template-hp-pie-5"),
        [6] = get_texture_info("template-hp-pie-6"),
        [7] = get_texture_info("template-hp-pie-7"),
        [8] = get_texture_info("template-hp-pie-8"),
    }
}
]]

--[[
    Note: If there are some functionalities you don't care
    to add such as palettes, you can freely remove the function
    for it, the only function you require is character_add
]]

local CSloaded = false
local function on_character_select_load()
    CT_MRV = _G.charSelect.character_add("Mr Virtual", {"I'm in your head."}, "5UP34, PK and Snart", {r = 200, g = 0, b = 0}, E_MODEL_MRV, CT_MARIO, MRV_LIFE_ICON, 1.4)
    _G.charSelect.character_add_caps(E_MODEL_MRV, MRV_CAPTABLE)
    --_G.charSelect.character_add_celebration_star(E_MODEL_MRV, E_MODEL_CUSTOM_STAR, TEX_CUSTOM_STAR_ICON)
    _G.charSelect.character_add_palette_preset(E_MODEL_MRV, PALETTE_CHAR)
    CSloaded = true
end

hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)

--------------------------------
-- Virtual Boy Lights Setting --
--------------------------------

local redToggleNum = _G.charSelect.add_option("Virtual Boy Filter", 0, 2, {"Off", "Mr.V Only", "On"}, {"Toggles if playing as MR. V", "Makes levels RED"})

local lightingChanged = false
local function update()
    if not CSloaded then return end
    local charNum = _G.charSelect.character_get_current_number()
    local redToggle = _G.charSelect.get_options_status(redToggleNum)
    if redToggle == nil then return end

    local VBColor = {r = 255, g = 0, b = 0}
    if redToggle > 0 and (redToggle == 2 or charNum == CT_MRV) then
        set_lighting_dir(1, 0x8000)
        set_lighting_dir(2, 0x8000)
        set_lighting_color(0, VBColor.r)
        set_lighting_color(1, VBColor.g)
        set_lighting_color(2, VBColor.b)
        set_vertex_color(0, VBColor.r)
        set_vertex_color(1, VBColor.g)
        set_vertex_color(2, VBColor.b)
        set_fog_color(0, VBColor.r*0.5)
        set_fog_color(1, VBColor.g*0.5)
        set_fog_color(2, VBColor.b*0.5)
        set_fog_intensity(1.05)
        lightingChanged = true
    elseif lightingChanged then
        set_lighting_dir(1, 0)
        set_lighting_dir(2, 0)
        set_lighting_color(0, 255)
        set_lighting_color(1, 255)
        set_lighting_color(2, 255)
        set_vertex_color(0, 255)
        set_vertex_color(1, 255)
        set_vertex_color(2, 255)
        set_fog_color(0, 255)
        set_fog_color(1, 255)
        set_fog_color(2, 255)
        set_fog_intensity(1)
        lightingChanged = false
    end
end

hook_event(HOOK_UPDATE, update)